/*
 * trap.c
 * $Id: trap.c,v 1.4 1998/02/02 16:31:20 moritz Exp $
 *
 * Funktionen fuer Signal-Behandlung fuer unix-connect
 * fuer den Fall eines Programmfehlers, um Datenverlusten
 * vorzubeugen
 *
 * init_trap() installiert Handler fuer Signale, die
 *             zu core fuehren
 *
 * trap_handler() ist der Handler, der eine Nachricht
 *	       ins Logfile und nach stderr sowie
 *	       den core in das logfile-Verzeichnis
 *	       schreibt 
 *
 * 9. Sep 1996 - Linux-fhige Version von Erich Oldendorf <olden@komtron.com>
 *
 * BUGS:
 *
 * Es schreibt noch nicht in ein Logfile.
 * syslog sollte benutzt werden.
 *
 *     $Log: trap.c,v $
 *     Revision 1.4  1998/02/02 16:31:20  moritz
 *     - Aenderungen der offiziellen Version bis 0.75b2 eingearbeitet
 *
 *     Revision 1.3  1996/09/19 15:37:05  moritz
 *     - trap fuer nur-SunOS eingeschaltet, fuer alle anderen aus
 *
 *     Revision 1.4  1996/09/06 11:09:32  matthias
 *     uberfluessige Leerzeilen entfernt;
 *
 *     Revision 1.3  1996/08/06 11:04:40  matthias
 *     'trap'-Funktion wird unter Linux praktisch ausgeschaltet;
 *
 */


#include "config.h"

#ifdef TRAPSIGNAL

#include <signal.h>
#include <stdio.h>
#include <malloc.h>
#ifdef HAS_UNISTD_H
#include <unistd.h>
#endif

#include "trap.h"
#include "ministat.h"

typedef void (*sighandler_t)(int);
sighandler_t s_handler(int sig, sighandler_t addr);

static signal_struct signallist[] = {
	{	SIGQUIT,	"SIGQUIT"	},
	{	SIGILL,		"SIGILL"	},
	{	SIGTRAP,	"SIGTRAP"	},
	{	SIGABRT,	"SIGABRT"	},
/*	{	SIGEMT,		"SIGEMT"	},	*/
	{	SIGFPE,		"SIGFPE"	},
	{	SIGBUS,		"SIGBUS"	},
	{	SIGSEGV,	"SIGSEGV"	},
/*	{	SIGSYS,		"SIGSYS"	},	*/
/*	{	SIGLOST,	"SIGLOST"	},	*/
	{	0,		"???"		}
};
static char *me;

void init_trap(char *progname) {
	signal_struct *sp;

	me = progname;
	for (sp = signallist; sp->no; sp++)
#ifdef BSD
		signal(sp->no, (sighandler_t)s_handler);
#else
	/* sighandler_t und __sighandler_t sind dasselbe,
	   aber das kapiert der gcc nicht und meckert. */
		signal(sp->no, (__sighandler_t)s_handler);
#endif
}

sighandler_t s_handler(int sig, sighandler_t addr) {
	signal_struct *sp;

	for (sp=signallist; sp->no && sp->no != sig; sp++);

	fprintf(stderr, "%s: caught signal %s\n", me, sp->name);
	fprintf(stderr, "total malloc'ed space: %d\n", mallinfo().arena);

	fprintf(stderr, "%s: dumping core to %s\n", me, logdir);
	chdir(logdir);
	signal(sig, SIG_DFL);
	kill(getpid(), sig);
	return 0; /* not reached, just for the compiler... */
}

#endif /* def TRAPSIGNAL */
