/* $Id: januslogin.c,v 1.14 2000/06/09 08:48:22 moritz Exp $ */
/*
 *  UNIX-Connect, a ZCONNECT(r) Transport and Gateway/Relay.
 *  Copyright (C) 1993-94  Martin Husemann
 *  Copyright (C) 1995-96  Christopher Creutzig
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * ------------------------------------------------------------------------
 * Eine deutsche Zusammenfassung zum Copyright dieser Programme sowie der
 * GNU General Public License finden Sie in der Datei README.1st.
 * ------------------------------------------------------------------------
 *
 *  Bugreports, suggestions for improvement, patches, ports to other systems
 *  etc. are welcome. Contact the maintainer by e-mail:
 *  christopher@nescio.foebud.org or snail-mail:
 *  Christopher Creutzig, Im Samtfelde 19, 33098 Paderborn
 *
 *  There is a mailing-list for user-support:
 *   unix-connect@mailinglisten.im-netz.de,
 *  write a mail with subject "Help" to
 *   unix-connect-request@mailinglisten.im-netz.de
 *  for instructions on how to join this list.
 */


/*
 * JANUS Login fuer UNIX
 *
 */

#include "config.h"
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#ifdef HAS_SYS_FCNTL_H
#include <sys/fcntl.h>
#endif
#ifdef HAS_FCNTL_H
#include <fcntl.h>
#endif
#ifndef NO_UNISTD_H
#include <unistd.h>
#endif
#ifdef HAS_BSD_DIRECT
#include <sys/dirent.h>
#include <sys/dir.h>
#else
#include <dirent.h>
#endif
#include "zconnect.h"
#include <ctype.h>
#include <sys/stat.h>
#include <setjmp.h>
#include <signal.h>
#include <dirent.h>
#include <time.h>
#include <errno.h>
#include "header.h"
#include "hd_def.h"
#include "hd_nam.h"
#include "version.h"
#include "track.h"
#include "line.h"
#include "boxstat.h"
#include "ministat.h"
#include "uulog.h"
#include "xprog.h"
#include "lib.h"
#include "locknames.h"

static jmp_buf timeout, nocarrier, abortsignal;
static char *tty = "/dev/tty";

FILE *deblogfile;
static char *logfile_name;

#define	WHO	"JANUS"

#define NAK	0x15
#define ACK	0x06

void handle_timeout(int code)
{
	longjmp(timeout, 1);
}

void handle_nocarrier(int code)
{
	longjmp(nocarrier, 1);
}

void handle_abortsignal(int code)
{
	longjmp(abortsignal, 1);
}
void getsln(char *p)
{
	int z;

	while (1) {
		z = getchar();
		if (z == '\r') break;
		if (isprint(z))
			*p++ = z;
	}
	*p = '\0';
	fputs("\r\n", stdout);
}

void getln(char *p)
{
	int z;

	while (1) {
		z = getchar();
		if (z == '\r') break;
		if (isprint(z))
			*p++ = z;
		putc(z, stdout);
	}
	*p = '\0';
	fputs("\r\n", stdout);
}

static void
findsysfile(const char *sysdir, char *sysname, char *filename)
{
	DIR *dir;
	struct dirent *this;
	int len;
	char buf[FILENAME_MAX], *p;

	len = strlen(sysname);
	strlwr(sysname);
	dir = opendir(sysdir);
	while ((this = readdir(dir)) != NULL) {
		strcpy(buf, this->d_name);
		p = strchr(buf, '.');
		if (p) *p = '\0';
		strlwr(buf);
		if (stricmp(buf, sysname) == 0) {
			strcpy(sysname, this->d_name);
			break;
		}
	}
	closedir(dir);
	sprintf(filename, "%s/%s", sysdir, sysname);
}

/* open private debug log file and redirect stderr to it. */
static char *privlog;
void open_debuglog(const char *file)
{
	privlog = (char *)dalloc(strlen(file) + 21);
	sprintf(privlog, "%s.%d", file, (int)getpid());
	deblogfile = fopen(privlog, "w+");
	if (!deblogfile) {
		deblogfile = fopen("/dev/null", "w");
		if (!deblogfile) {
			printf("Sorry - kann Logfile nicht schreiben...\n");
			exit(10);
		}
	}
	
	/* stderr auf Debug-Logfile umleiten. */
	fflush(stderr);
	dup2(fileno(deblogfile), 2);
}

/* append the log file file to the main log file */
void close_debuglog(const char *file)
{
	FILE *f;
	const int bufsiz = 1024;
	char buf[bufsiz];
	int n;
	struct tm *tmp;
	time_t t;

	fseek(deblogfile, 0, SEEK_SET);
	f = fopen(file, "a");
	if (f == NULL)
	{
		fprintf(deblogfile, "januslogin: cant open \"%s\" for append, l"
			"eaving debug log file alone\n", file);
	}
	else {
		t = time(NULL);
		fprintf(f,"\n----------------------\n"
				"START OF SESSION pid=%d\n",
				(int)getpid());
		for(;;) {
			n = fread(buf, 1, bufsiz, deblogfile);
			if (n > 0) 
				fwrite(buf, 1, n, f);
			else
				break;
		}
		tmp = localtime(&t);
		strftime(buf, bufsiz-1, "%Y-%m-%d  %H:%M:%S", tmp);
		fprintf(f, "\nSESSION ENDS %s\n\n", buf);
		fclose(f);
		unlink(privlog);
		dfree(privlog);
	}
	fclose(deblogfile);
}

void reject(char *prog, char *sysname, char *passwd)
{
	fputs("Netzzugriff verweigert.\r\n", stdout);
	logfile(INCOMING, WHO, sysname, "-", "Falsches Passwort: %s\n", passwd);
	fprintf(deblogfile, "%s: Fehlversuch (System %s Passwort %s)\n", prog, sysname, passwd);
	lock_device(0, tty);
	close_debuglog(logfile_name);
	exit(0);
}

/*
 *   Hauptprogramm, Gesamtstruktur
 */ 
int main(int argc, char **argv)
{
	char sysname[FILENAME_MAX];
	char passwd[80];
	char filename[FILENAME_MAX];
	char lockname[FILENAME_MAX];
	char tmpname[FILENAME_MAX];
	char outname[FILENAME_MAX];
	char lastcallname[FILENAME_MAX];
	char *arcer;
	char *arcerin, *transfer, sernr[6];
	char arcer_out_suffix_for_outfile[10];
	int i, j, z, serchk;
	volatile int dummy_sendfile_created;
	int do_delete_tmpdir;
	size_t bytes_in, bytes_out, bytes_dummy;
	FILE *f;
	header_p hd, p;
	struct stat stat_buf;
	DIR *dir;
#ifdef HAS_BSD_DIRECT
        struct direct *ent;
#else
        struct dirent *ent;
#endif
	char *cp;

	
	dummy_sendfile_created = 0;
	/*
	 *  Konfiguration einlesen.
	 */
	minireadstat();
	
	/*  Da dieses Programm Login-Shell ist, muss die Fehlerausgabe
	 *  irgendwo hin dirigiert werden...
	 *  Wir erzeugen ein Logfile je Prozess und haengen sie am
	 *  Ende wieder aneinander. Wir wollen zumindest die Moeglichkeit
	 *  haben, die Debug-Informationen von Transferprotokollen zu
	 *  analysieren.
	 */
	logfile_name = (char *) malloc(strlen(logdir)+20);
	sprintf(logfile_name, "%s/januslogin.log", logdir);
	open_debuglog(logfile_name);
	
	/*
	 *  Verschiedene Abbruch-Gruende vorbereiten
	 */
	if (setjmp(timeout)) {
		fputs("\nABBRUCH: Timeout\n", deblogfile);
		fclose(deblogfile);
		logfile(INCOMING, WHO, "-", "-", "TIMEOUT\n");
		close_debuglog(logfile_name);
		return 1;
	}
	if (setjmp(nocarrier)) {
		fputs("\nABBRUCH: Gegenstelle hat aufgelegt!\n", deblogfile);
		fclose(deblogfile);
		logfile(INCOMING, WHO, "-", "-", "NO CARRIER\n");
		close_debuglog(logfile_name);
		return 2;
	}
	if (setjmp(abortsignal)) {
		fputs("\nABBRUCH: Abbruchsignal vom Betriebssystem\n", deblogfile);
		fclose(deblogfile);
		logfile(INCOMING, WHO, "-", "-", "ABORT SIGNALLED\n");
		close_debuglog(logfile_name);
		return 127;
	}
	signal(SIGHUP, handle_nocarrier);
	signal(SIGALRM, handle_timeout);
	signal(SIGINT, handle_abortsignal);
	signal(SIGTERM, handle_abortsignal);

	/*
	 *  Hier beginnt der eigentliche Datenaustausch
	 */
	set_rawmode(fileno(stdin));
	set_rawmode(fileno(stdout));

	tty = ttyname(0);
	lock_device(1, tty);

	alarm(45);
ask_sysname:
	sleep(1); fflush(stdin);
	fputs("Systemname: ", stdout); fflush(stdout);
	getln(sysname);
	if (!sysname[0]) goto ask_sysname;	/* z.B. bei Ctrl-X */
	strlwr(sysname);
	if (strcmp(sysname, "zerberus") == 0) goto ask_sysname;
	if (strcmp(sysname, "janus") == 0) goto ask_sysname; /* Wenn schon, denn schon */
ask_passwort:
	sleep(1); fflush(stdin);
	fputs("Passwort: ", stdout); fflush(stdout);
	getsln(passwd);
	if (stricmp(passwd, "zerberus") == 0) goto ask_sysname;
	if (stricmp(passwd, "janus") == 0) goto ask_sysname; /* Hier auch nochmal */
	if (stricmp(sysname, passwd) == 0) goto ask_passwort;
	alarm(0);

	findsysfile(systemedir, sysname, filename);
	f = fopen(filename, "rb");
	if (!f) {
		fprintf(deblogfile, "Unbekanntes System: %s\n", sysname);
		reject(argv[0], sysname, passwd);
	}
	hd = rd_para(f);
	fclose(f);
	p = find(HD_SYS, hd);
	if (p) {
		char buf[FILENAME_MAX], *c;
		strcpy(buf, sysname);
		c = strchr(buf, '.');
		if (c) *c = '\0';
		if (stricmp(buf, p->text) != 0) {
			fprintf(deblogfile, "Unbekanntes System: %s\n", sysname);
			reject(argv[0], sysname, passwd);
		}
	} else {
		fprintf(deblogfile, "Illegale Systemdatei: Kein "HN_SYS": Header: %s\n", filename);
		reject(argv[0], sysname, passwd);
	}

	p = find(HD_PASSWD, hd);
	if (!p || stricmp(p->text, passwd) != 0) {
		fputs("Passwort falsch:\n", deblogfile);
		reject(argv[0], sysname, passwd);
	}

	p = find(HD_ARCEROUT, hd);
	if (!p) {
		fputs("Kein ARCer (ausgehend) definiert!\n", deblogfile);
		reject(argv[0], sysname, passwd);
	}
	arcer = p->text;
	strlwr(arcer);
	strncpy(arcer_out_suffix_for_outfile, arcer, 9);
	arcer_out_suffix_for_outfile[9] = 0;
	if (strncmp(arcer_out_suffix_for_outfile, "non", 3) == 0) {
		strcpy(arcer_out_suffix_for_outfile, "non");
		arcer = "nar";
	}

	p = find(HD_ARCERIN, hd);
	if (!p) {
		fputs("Kein ARCer (eingehend) definiert!\n", deblogfile);
		reject(argv[0], sysname, passwd);
	}
	arcerin = p->text;
	strlwr(arcerin);

	p = find(HD_PROTO, hd);
	if (!p) {
		fputs("Kein Uebertragungsprotokoll definiert!\n", deblogfile);
		reject(argv[0], sysname, passwd);
	}
	transfer = p->text;
	strlwr(transfer);

	fputs("Running ARC....\r\n"
	     "Running ARC....\r\n"
	     "Running ARC....\r\n"
	     "Running ARC....\r\n", stdout);
	fprintf(deblogfile, "%s: System %s erfolgreich eingelogt [%s]\n", argv[0], sysname, tty);
	logfile(INCOMING, WHO, sysname, tty, "Login erfolgreich\n");

	sprintf(tmpname, "%s/%s.%d.dir", netcalldir, sysname, getpid());
	mkdir(tmpname, 0777);

	sprintf(lastcallname, "%s/%s.netcall", netcalldir, sysname);

	if (setjmp(timeout)) {
		fputs("\nABBRUCH: Timeout\n", deblogfile);
		logfile(INCOMING, WHO, "-", "-", "TIMEOUT\n");
		close_debuglog(logfile_name);
		return 1;
	}
	if (setjmp(nocarrier)) {
		fputs("\nABBRUCH: Gegenstelle hat aufgelegt!\n", deblogfile);
		logfile(INCOMING, WHO, "-", "-", "NO CARRIER\n");
		close_debuglog(logfile_name);
		return 2;
	}

	chdir(tmpname);
	sprintf(outname, "%s/called.%s", tmpname, arcer_out_suffix_for_outfile);
	sprintf(filename, "%s/%s.%s", netcalldir, sysname, arcer);
	sprintf(lockname, "%s/%s/" PREARC_LOCK, netcalldir, sysname);
	if (access(filename, R_OK) != 0) {
		f = fopen(outname, "wb");
		if (!f) {
			fprintf(deblogfile, "Kann Netcall %s nicht erzeugen\n", outname);
			close_debuglog(logfile_name);
			return 1;
		}
		fputs("\r\n", f);
		fclose(f);
		dummy_sendfile_created = 1;
	} else {
		if (waitnolock(lockname, 180)) {
			logfile(INCOMING, WHO, sysname, "-", "Prearc LOCK: %s\n", lockname);
			fprintf(deblogfile, "Prearc LOCK: %s\n", lockname);
			close_debuglog(logfile_name);
			return 1;
		}
		if(rename(filename, outname)) {
			fprintf(deblogfile, "Umbenennen: %s -> %s fehlgeschlagen\n", filename, outname);
			close_debuglog(logfile_name);
			return 1;
		}
	}
	sprintf(filename, "caller.%s", arcer);
	sprintf(outname, "called.%s", arcer_out_suffix_for_outfile);

	if (setjmp(timeout)) {
		fputs("\nABBRUCH: Timeout\n", deblogfile);
		logfile(INCOMING, WHO, "-", "-", "TIMEOUT\n");
		goto cleanup;
	}
	if (setjmp(nocarrier)) {
		fputs("\nABBRUCH: Gegenstelle hat aufgelegt!\n", deblogfile);
		logfile(INCOMING, WHO, "-", "-", "NO CARRIER\n");
		goto cleanup;
	}
	if (setjmp(abortsignal)) {
		fputs("\nABBRUCH: Abrruchsignal vom Betriebssystem!\n", deblogfile);
		logfile(INCOMING, WHO, "-", "-", "ABORT SIGNALLED\n");
		goto cleanup;
	}

	/*
	 *  Man glaubt es kaum, aber einige Programme schaffen es nicht,
	 *  ein Packzeit von 0 Sekunden der Gegenseite zu verstehen...
	 */
	sleep(3);

	alarm(30);	
	for (i=1; i<7; i++) {
		putchar(NAK); fflush(stdout);
		z = 0;
		for (j=0; j<4; j++) 
			sernr[j] = toupper(getchar());
		serchk = getchar();
/* Die Pruefsumme der Seriennummer zu pruefen, ist eigentlich unnoetig. ccr */
/*		if ((sernr[0]+sernr[1]+sernr[2]+sernr[3]) == serchk & 0x0ff)*/ {
			alarm(0);
			putchar(ACK); fflush(stdout);
			sernr[4] = '\0';
			logfile(INCOMING, WHO, "-", "-", "Seriennummer: %s\n", sernr);
			fflush(deblogfile);
			if (recvfile(transfer, filename)) {
				logfile(INCOMING, WHO, filename, transfer, "Empfang fehlgeschlagen\n");
				fprintf(deblogfile, "Empfang der Daten fehlgeschlagen!\n");
				break;
			}
			fflush(deblogfile);
			if (sendfile(transfer, outname)) {
				logfile(INCOMING, WHO, outname, transfer, "Versand fehlgeschlagen\n");
				fprintf(deblogfile, "Versand der Daten fehlgeschlagen\n");
				break;
			}
			bytes_out = 0;
			if (stat(outname, &stat_buf) == 0)
				bytes_out = stat_buf.st_size;

			/* Fertig, jetzt auf jeden Fall weitermachen,
			 * Modem auflegen */
			signal(SIGHUP, SIG_IGN);
			signal(SIGTERM, SIG_IGN);
			signal(SIGINT, SIG_IGN);
			fclose(stdin);
			hangup(fileno(stdout));
			fclose(stdout);
			lock_device(0, tty);

			/* Netcall war erfolgreich, also Daten ins backup */
			logfile(INCOMING, WHO, sysname, "-", "Netcall erfolgreich\n");
			if (backoutdir && !dummy_sendfile_created) {
				char backoutname[FILENAME_MAX], *shortname;

				shortname = strrchr(outname, '/');
				if (!shortname) shortname = outname;
				sprintf(backoutname, "%s/%s.%s.%ld", backoutdir,
					sysname, shortname, (long)time(NULL));
				logfile(INCOMING, WHO, sysname, "-", 
					"BackOut: %s -> %s\n",
					outname, backoutname);
				remove(backoutname);
				rename(outname, backoutname);
			} else {
				remove(outname);
			}

			/* Flag Datei fuer letzten erfolgreichen Anruf
			 * updaten
			 */
			j = open(lastcallname, O_CREAT|O_WRONLY|O_TRUNC, 0775);
			close(j);

			/*
			 * Und empfangene Daten (im Hintergrund) einlesen,
			 * das Modem wird sofort wieder freigegeben.
			 */
			if (fork() == 0) {
				/* Ich bin child */
				fflush(deblogfile);
				import_all(arcerin, sysname, 0,
					&bytes_in, &bytes_dummy);
				logfile(INCOMING, WHO, sysname, "-", 
					"Bytes gepackt: in %lu, out %lu\n",
					bytes_in, bytes_out);

				chdir ("/");
				rmdir(tmpname);
				close_debuglog(logfile_name);
			}

			return 0;
		}
	}

cleanup:
	alarm(0);
	fprintf(deblogfile, "Netcall fehlgeschlagen\n");
	logfile(INCOMING, WHO, "-", "-", "Netcall abgebrochen\n");
	lock_device(0, tty);

	/* Falls wir Daten empfangen haben, werden diese weggeworfen. */
	/* Falls sie unter anderen/m Namen kamen, passiert das erst unten. */
	do_delete_tmpdir = 1;
	remove(filename); 

	/* Jetzt muessen die Daten zurueck aus dem temporaeren Verzeichnis
	 * ins Haupt-Netcall-Verzeichnis - falls wir nicht einen dummy
	 * erzeugt hatten.
	 */
	sprintf(filename, "%s/%s.%s", netcalldir, sysname, arcer);
	sprintf(outname, "%s/called.%s", tmpname, arcer_out_suffix_for_outfile);
	if (dummy_sendfile_created) {
		remove(outname);
	}
	else {
		if (waitnolock(lockname, 180)) {
			logfile(INCOMING, WHO, sysname, "-", "NC Abbruch, zu sendende Daten in %s konnten nicht zurueckkopiert werden wegen Prearc LOCK: %s\n",
					tmpname, lockname);
			fprintf(deblogfile, "Prearc LOCK: %s\n", lockname);
			fclose(deblogfile);
			return 1;
		}
		if (access(filename, R_OK) != 0) {
			/*
			 * Der einfache Fall: es ist noch nichts neues da,
			 * wir koennen die ARC Datei an die urspruengliche
			 * Stelle zurueckschieben.
			 * rename() sollte nicht benutzt werden, da die Datei
			 * hinter dem access() erzeugt worden sein koennte.
			 */
			if (link(outname, filename) != 0) {
				logfile(INCOMING, WHO, sysname, "-",
					"NC Abbruch, Datei %s konnte nicht zur"
					"ueck nach %s gelinkt werden: %s (Dat"
					"en werden wieder ausgepackt)\n",
					outname, filename, sys_errlist[errno]);
				/* outname nicht loeschen (s.u.) */
			}
			else {
				/* link() hat funktioniert */
				if (unlink(outname) != 0) {
					logfile(INCOMING, WHO, sysname, "-",
						"NC Abbruch: Konnte Archiv \"%s"
						"\" nicht loeschen, nachem es "
						"bereits nach \"%s\" gelinkt w"
						"urde (Daten koennten dadurch "
						"doppelt gesendet werden): %s\n",
						outname, filename, 
						sys_errlist[errno]);
				}
			}
		} 
		if (access(outname, R_OK) == 0) {
			/*
			 * Wenn die Datei, die wir senden wollten, jetzt noch
			 * da ist, muessen wir sie im richtigen Netcallver-
			 * zeichnis wieder auspacken, damit sie wieder gearct
			 * werden.
			 */
			if (chdir(netcalldir) != 0) {
				logfile(INCOMING, WHO, sysname, "-",
					"NC Abbruch: Kann nicht in Verzeichnis "
					"\"%s\" wechseln (lasse Netcalldatei "
					"\"%s\" liegen!): %s\n", netcalldir,
					outname, sys_errlist[errno]);
			}
			else if (chdir(sysname) != 0) {
				logfile(INCOMING, WHO, sysname, "-",
					"NC Abbruch: Kann nicht in Verzeichnis "
					"\"%s\" wechseln (lasse Netcalldatei "
					"\"%s\" liegen!): %s\n", sysname,
					outname, sys_errlist[errno]);
			} else if (call_auspack(arcer, outname) != 0) {
				logfile(INCOMING, WHO, sysname, "-",
					"NC Abbruch: Kann misslungene Netcall-"
					"Sende-Daten \"%s\" nicht wieder auspa"
					"cken (lasse Netcalldatei liegen!): "
					"%s\n", outname, sys_errlist[errno]);
			} else if (remove(outname) != 0) {
				logfile(INCOMING, WHO, sysname, "-",
					"NC Abbruch: Nach erfolgreichem wieder-"
					"auspacken kann das Sende-Archiv \"%s\""
					" nicht geloescht werden: %s\n",
					outname, sys_errlist[errno]);
			}
		}
	}
	chdir("/");
	if (do_delete_tmpdir && (cp = strchr(tmpname, 0)) != NULL) {
		if ((dir = opendir(tmpname)) == NULL) {
			perror(tmpname);
			exit(1);
		}
		*cp++ = '/';
		while ((ent = readdir(dir)) != NULL) {
			strcpy(cp, ent->d_name);
			if (*cp == '.' && (cp[1] == 0 || (cp[1] == '.' && cp[2] == 0)))
				continue;
			remove(tmpname);
		}
		closedir(dir);
		*(--cp) = 0;
		rmdir(tmpname);
	}

	close_debuglog(logfile_name);
	dfree(logfile_name);

	return 0;
}
